@@ -6,10 +6,14 @@
 
 from __future__ import absolute_import, division, print_function
 
+import sys
+
 import pytest
 
 from six.moves import cStringIO as StringIO
 
+from structlog.stdlib import _NAME_TO_LEVEL
+
 
 @pytest.fixture
 def sio():
@@ -24,8 +28,22 @@ def event_dict():
     """
     An example event dictionary with multiple value types w/o the event itself.
     """
+
     class A(object):
         def __repr__(self):
-            return "<A(\o/)>"
+            return r"<A(\o/)>"
 
     return {"a": A(), "b": [3, 4], "x": 7, "y": "test", "z": (1, 2)}
+
+
+@pytest.fixture(
+    name="stdlib_log_method",
+    params=[m for m in _NAME_TO_LEVEL if m != "notset"],
+)
+def fixture_stdlib_log_methods(request):
+    return request.param
+
+
+collect_ignore = []
+if sys.version_info[:2] < (3, 7):
+    collect_ignore.append("tests/test_contextvars.py")
