@@ -3,10 +3,11 @@ import sys
 import uuid
 
 import flask
-import structlog
 
 from some_module import some_function
 
+import structlog
+
 
 logger = structlog.get_logger()
 app = flask.Flask(__name__)
@@ -14,9 +15,7 @@ app = flask.Flask(__name__)
 
 @app.route("/login", methods=["POST", "GET"])
 def some_route():
-    log = logger.new(
-        request_id=str(uuid.uuid4()),
-    )
+    log = logger.new(request_id=str(uuid.uuid4()))
     # do something
     # ...
     log.info("user logged in", user="test-user")
@@ -27,17 +26,16 @@ def some_route():
     # ...
     return "logged in!"
 
+
 if __name__ == "__main__":
     logging.basicConfig(
-        format="%(message)s",
-        stream=sys.stdout,
-        level=logging.INFO,
+        format="%(message)s", stream=sys.stdout, level=logging.INFO
     )
     structlog.configure(
         processors=[
             structlog.processors.KeyValueRenderer(
-                key_order=["event", "request_id"],
-            ),
+                key_order=["event", "request_id"]
+            )
         ],
         context_class=structlog.threadlocal.wrap_dict(dict),
         logger_factory=structlog.stdlib.LoggerFactory(),
