@@ -8,35 +8,22 @@ Structured logging for Python.
 
 from __future__ import absolute_import, division, print_function
 
-from structlog import (
-    dev,
-    processors,
-    stdlib,
-    threadlocal,
-)
-from structlog._base import (
-    BoundLoggerBase,
-)
+from structlog import dev, processors, stdlib, testing, threadlocal
+from structlog._base import BoundLoggerBase
 from structlog._config import (
     configure,
     configure_once,
-    getLogger,
+    get_config,
     get_logger,
+    getLogger,
+    is_configured,
     reset_defaults,
     wrap_logger,
 )
-from structlog.exceptions import (
-    DropEvent,
-)
-from structlog._generic import (
-    BoundLogger
-)
-from structlog._loggers import (
-    PrintLogger,
-    PrintLoggerFactory,
-    ReturnLogger,
-    ReturnLoggerFactory,
-)
+from structlog._generic import BoundLogger
+from structlog._loggers import PrintLogger, PrintLoggerFactory
+from structlog.exceptions import DropEvent
+from structlog.testing import ReturnLogger, ReturnLoggerFactory
 
 
 try:
@@ -45,17 +32,17 @@ except ImportError:  # pragma: nocover
     twisted = None
 
 
-__version__ = "17.2.0"
+__version__ = "20.1.0"
 
 __title__ = "structlog"
 __description__ = "Structured Logging for Python"
-__uri__ = "http://www.structlog.org/"
+__uri__ = "https://www.structlog.org/"
 
 __author__ = "Hynek Schlawack"
 __email__ = "hs@ox.cx"
 
 __license__ = "MIT or Apache License, Version 2.0"
-__copyright__ = "Copyright (c) 2013 {0}".format(__author__)
+__copyright__ = "Copyright (c) 2013 " + __author__
 
 
 __all__ = [
@@ -68,12 +55,15 @@ __all__ = [
     "ReturnLoggerFactory",
     "configure",
     "configure_once",
-    "getLogger",
-    "get_logger",
     "dev",
+    "getLogger",
+    "get_config",
+    "get_logger",
+    "is_configured",
     "processors",
     "reset_defaults",
     "stdlib",
+    "testing",
     "threadlocal",
     "twisted",
     "wrap_logger",
