@@ -10,9 +10,9 @@ from pretend import raiser, stub
 
 from structlog._base import BoundLoggerBase
 from structlog._config import _CONFIG
-from structlog._loggers import ReturnLogger
 from structlog.exceptions import DropEvent
 from structlog.processors import KeyValueRenderer
+from structlog.testing import ReturnLogger
 
 
 def build_bl(logger=None, processors=None, context=None):
@@ -30,9 +30,9 @@ class TestBinding(object):
     def test_repr(self):
         bl = build_bl(processors=[1, 2, 3], context={})
 
-        assert (
-            "<BoundLoggerBase(context={}, processors=[1, 2, 3])>"
-        ) == repr(bl)
+        assert ("<BoundLoggerBase(context={}, processors=[1, 2, 3])>") == repr(
+            bl
+        )
 
     def test_binds_independently(self):
         """
@@ -40,42 +40,80 @@ class TestBinding(object):
         """
         b = build_bl(processors=[KeyValueRenderer(sort_keys=True)])
         b = b.bind(x=42, y=23)
-        b1 = b.bind(foo='bar')
-        b2 = b.bind(foo='qux')
+        b1 = b.bind(foo="bar")
+        b2 = b.bind(foo="qux")
+
         assert b._context != b1._context != b2._context
 
     def test_new_clears_state(self):
         b = build_bl()
         b = b.bind(x=42)
-        assert 42 == b._context['x']
+
+        assert 42 == b._context["x"]
+
         b = b.bind()
-        assert 42 == b._context['x']
+
+        assert 42 == b._context["x"]
+
         b = b.new()
-        assert 'x' not in b._context
+
+        assert "x" not in b._context
 
     def test_comparison(self):
         b = build_bl()
+
         assert b == b.bind()
         assert b is not b.bind()
         assert b != b.bind(x=5)
-        assert b != 'test'
+        assert b != "test"
 
     def test_bind_keeps_class(self):
         class Wrapper(BoundLoggerBase):
             pass
+
         b = Wrapper(None, [], {})
+
         assert isinstance(b.bind(), Wrapper)
 
     def test_new_keeps_class(self):
         class Wrapper(BoundLoggerBase):
             pass
+
         b = Wrapper(None, [], {})
+
         assert isinstance(b.new(), Wrapper)
 
     def test_unbind(self):
-        b = build_bl().bind(x=42, y=23).unbind('x', 'y')
+        """
+        unbind() removes keys from context.
+        """
+        b = build_bl().bind(x=42, y=23).unbind("x", "y")
+
         assert {} == b._context
 
+    def test_unbind_fail(self):
+        """
+        unbind() raises KeyError if the key is missing.
+        """
+        with pytest.raises(KeyError):
+            build_bl().bind(x=42, y=23).unbind("x", "z")
+
+    def test_try_unbind(self):
+        """
+        try_unbind() removes keys from context.
+        """
+        b = build_bl().bind(x=42, y=23).try_unbind("x", "y")
+
+        assert {} == b._context
+
+    def test_try_unbind_fail(self):
+        """
+        try_unbind() does nothing if the key is missing.
+        """
+        b = build_bl().bind(x=42, y=23).try_unbind("x", "z")
+
+        assert {"y": 23} == b._context
+
 
 class TestProcessing(object):
     def test_event_empty_string(self):
@@ -94,18 +132,20 @@ class TestProcessing(object):
         BoundLoggerBase._process_event() gets called before relaying events
         to wrapped loggers.
         """
+
         def chk(_, __, event_dict):
             assert b._context is not event_dict
-            return ''
+            return ""
 
         b = build_bl(processors=[chk])
-        assert (('',), {}) == b._process_event('', 'event', {})
-        assert 'event' not in b._context
+
+        assert (("",), {}) == b._process_event("", "event", {})
+        assert "event" not in b._context
 
     def test_chain_does_not_swallow_all_exceptions(self):
         b = build_bl(processors=[raiser(ValueError)])
         with pytest.raises(ValueError):
-            b._process_event('', 'boom', {})
+            b._process_event("", "boom", {})
 
     def test_last_processor_returns_string(self):
         """
@@ -113,34 +153,29 @@ class TestProcessing(object):
         returned.
         """
         logger = stub(msg=lambda *args, **kw: (args, kw))
-        b = build_bl(logger, processors=[lambda *_: 'foo'])
-        assert (
-            (('foo',), {}) ==
-            b._process_event('', 'foo', {})
-        )
+        b = build_bl(logger, processors=[lambda *_: "foo"])
+
+        assert (("foo",), {}) == b._process_event("", "foo", {})
 
     def test_last_processor_returns_tuple(self):
         """
         If the final processor returns a tuple, it is just passed through.
         """
         logger = stub(msg=lambda *args, **kw: (args, kw))
-        b = build_bl(logger, processors=[lambda *_: (('foo',),
-                                                     {'key': 'value'})])
-        assert (
-            (('foo',), {'key': 'value'}) ==
-            b._process_event('', 'foo', {})
+        b = build_bl(
+            logger, processors=[lambda *_: (("foo",), {"key": "value"})]
         )
 
+        assert (("foo",), {"key": "value"}) == b._process_event("", "foo", {})
+
     def test_last_processor_returns_dict(self):
         """
         If the final processor returns a dict, ``(), the_dict`` is returned.
         """
         logger = stub(msg=lambda *args, **kw: (args, kw))
-        b = build_bl(logger, processors=[lambda *_: {'event': 'foo'}])
-        assert (
-            ((), {'event': 'foo'}) ==
-            b._process_event('', 'foo', {})
-        )
+        b = build_bl(logger, processors=[lambda *_: {"event": "foo"}])
+
+        assert ((), {"event": "foo"}) == b._process_event("", "foo", {})
 
     def test_last_processor_returns_unknown_value(self):
         """
@@ -150,11 +185,9 @@ class TestProcessing(object):
         logger = stub(msg=lambda *args, **kw: (args, kw))
         b = build_bl(logger, processors=[lambda *_: object()])
         with pytest.raises(ValueError) as exc:
-            b._process_event('', 'foo', {})
+            b._process_event("", "foo", {})
 
-        assert (
-            exc.value.args[0].startswith("Last processor didn't return")
-        )
+        assert exc.value.args[0].startswith("Last processor didn't return")
 
 
 class TestProxying(object):
@@ -165,4 +198,5 @@ class TestProxying(object):
         """
         b = build_bl(processors=[raiser(DropEvent), raiser(ValueError)])
         b._proxy_to_logger("", None, x=5)
-        assert (("", "") == capsys.readouterr())
+
+        assert ("", "") == capsys.readouterr()
