@@ -6,6 +6,8 @@
 
 from __future__ import absolute_import, division, print_function
 
+import pickle
+
 import pytest
 import six
 
@@ -39,9 +41,7 @@ def styles(cr):
 @pytest.fixture
 def padded(styles):
     return (
-        styles.bright +
-        dev._pad("test", dev._EVENT_WIDTH) +
-        styles.reset + " "
+        styles.bright + dev._pad("test", dev._EVENT_WIDTH) + styles.reset + " "
     )
 
 
@@ -58,7 +58,7 @@ class TestConsoleRenderer(object):
         colorama is missing.
         """
         with pytest.raises(SystemError) as e:
-            dev.ConsoleRenderer()
+            dev.ConsoleRenderer(colors=True)
 
         assert (
             "ConsoleRenderer with `colors=True` requires the colorama package "
@@ -79,18 +79,38 @@ class TestConsoleRenderer(object):
         """
         rv = cr(None, None, {"event": "test", "timestamp": 42})
 
-        assert (
-            styles.timestamp + "42" + styles.reset + " " + unpadded
-        ) == rv
+        assert (styles.timestamp + "42" + styles.reset + " " + unpadded) == rv
+
+    def test_event_stringified(self, cr, styles, unpadded):
+        """
+        Event is cast to string.
+        """
+        not_a_string = Exception("test")
+
+        rv = cr(None, None, {"event": not_a_string})
+
+        assert unpadded == rv
+
+    @pytest.mark.skipif(not six.PY2, reason="Problem only exists on Python 2.")
+    @pytest.mark.parametrize("s", [u"\xc3\xa4".encode("utf-8"), u"ä", "ä"])
+    def test_event_py2_only_stringify_non_strings(self, cr, s, styles):
+        """
+        If event is a string type already, leave it be on Python 2. Running
+        str() on unicode strings with non-ascii characters raises an error.
+        """
+        rv = cr(None, None, {"event": s})
+
+        assert styles.bright + s + styles.reset == rv
 
     def test_level(self, cr, styles, padded):
         """
         Levels are rendered aligned, in square brackets, and color coded.
         """
-        rv = cr(None, None, {
-            "event": "test", "level": "critical", "foo": "bar"
-        })
+        rv = cr(
+            None, None, {"event": "test", "level": "critical", "foo": "bar"}
+        )
 
+        # fmt: off
         assert (
             "[" + dev.RED + styles.bright +
             dev._pad("critical", cr._longest_level) +
@@ -99,6 +119,7 @@ class TestConsoleRenderer(object):
             styles.kv_key + "foo" + styles.reset + "=" +
             styles.kv_value + "bar" + styles.reset
         ) == rv
+        # fmt: on
 
     def test_init_accepts_overriding_levels(self, styles, padded):
         """
@@ -113,10 +134,11 @@ class TestConsoleRenderer(object):
         )
 
         # this would blow up if the level_styles override failed
-        rv = cr(None, None, {
-            "event": "test", "level": "MY_OH_MY", "foo": "bar"
-        })
+        rv = cr(
+            None, None, {"event": "test", "level": "MY_OH_MY", "foo": "bar"}
+        )
 
+        # fmt: off
         assert (
             "[" + dev.RED + styles.bright +
             dev._pad("MY_OH_MY", cr._longest_level) +
@@ -125,6 +147,7 @@ class TestConsoleRenderer(object):
             styles.kv_key + "foo" + styles.reset + "=" +
             styles.kv_value + "bar" + styles.reset
         ) == rv
+        # fmt: on
 
     def test_logger_name(self, cr, styles, padded):
         """
@@ -132,22 +155,22 @@ class TestConsoleRenderer(object):
         """
         rv = cr(None, None, {"event": "test", "logger": "some_module"})
 
+        # fmt: off
         assert (
             padded +
             "[" + dev.BLUE + styles.bright +
             "some_module" +
             styles.reset + "] "
         ) == rv
+        # fmt: on
 
     def test_key_values(self, cr, styles, padded):
         """
         Key-value pairs go sorted alphabetically to the end.
         """
-        rv = cr(None, None, {
-            "event": "test",
-            "key": "value",
-            "foo": "bar",
-        })
+        rv = cr(None, None, {"event": "test", "key": "value", "foo": "bar"})
+
+        # fmt: off
         assert (
             padded +
             styles.kv_key + "foo" + styles.reset + "=" +
@@ -157,6 +180,7 @@ class TestConsoleRenderer(object):
             styles.kv_value + "value" +
             styles.reset
         ) == rv
+        # fmt: on
 
     def test_exception(self, cr, padded):
         """
@@ -164,38 +188,28 @@ class TestConsoleRenderer(object):
         """
         exc = "Traceback:\nFake traceback...\nFakeError: yolo"
 
-        rv = cr(None, None, {
-            "event": "test",
-            "exception": exc
-        })
+        rv = cr(None, None, {"event": "test", "exception": exc})
 
-        assert (
-            padded + "\n" + exc
-        ) == rv
+        assert (padded + "\n" + exc) == rv
 
     def test_stack_info(self, cr, padded):
         """
         Stack traces are rendered after a new line.
         """
         stack = "fake stack"
-        rv = cr(None, None, {
-            "event": "test",
-            "stack": stack
-        })
+        rv = cr(None, None, {"event": "test", "stack": stack})
 
-        assert (
-            padded + "\n" + stack
-        ) == rv
+        assert (padded + "\n" + stack) == rv
 
     def test_pad_event_param(self, styles):
         """
         `pad_event` parameter works.
         """
-        rv = dev.ConsoleRenderer(42, dev._has_colorama)(None, None, {
-            "event": "test",
-            "foo": "bar"
-        })
+        rv = dev.ConsoleRenderer(42, dev._has_colorama)(
+            None, None, {"event": "test", "foo": "bar"}
+        )
 
+        # fmt: off
         assert (
             styles.bright +
             dev._pad("test", 42) +
@@ -203,6 +217,7 @@ class TestConsoleRenderer(object):
             styles.kv_key + "foo" + styles.reset + "=" +
             styles.kv_value + "bar" + styles.reset
         ) == rv
+        # fmt: on
 
     def test_everything(self, cr, styles, padded):
         """
@@ -211,17 +226,22 @@ class TestConsoleRenderer(object):
         exc = "Traceback:\nFake traceback...\nFakeError: yolo"
         stack = "fake stack trace"
 
-        rv = cr(None, None, {
-            "event": "test",
-            "exception": exc,
-            "key": "value",
-            "foo": "bar",
-            "timestamp": "13:13",
-            "logger": "some_module",
-            "level": "error",
-            "stack": stack,
-        })
+        rv = cr(
+            None,
+            None,
+            {
+                "event": "test",
+                "exception": exc,
+                "key": "value",
+                "foo": "bar",
+                "timestamp": "13:13",
+                "logger": "some_module",
+                "level": "error",
+                "stack": stack,
+            },
+        )
 
+        # fmt: off
         assert (
             styles.timestamp + "13:13" + styles.reset +
             " [" + styles.level_error + styles.bright +
@@ -240,6 +260,7 @@ class TestConsoleRenderer(object):
             "\n" + stack + "\n\n" + "=" * 79 + "\n" +
             "\n" + exc
         ) == rv
+        # fmt: on
 
     def test_colorama_colors_false(self):
         """
@@ -247,25 +268,26 @@ class TestConsoleRenderer(object):
         """
         plain_cr = dev.ConsoleRenderer(colors=False)
 
-        rv = plain_cr(None, None, {
-            "event": "event", "level": "info", "foo": "bar"
-        })
+        rv = plain_cr(
+            None, None, {"event": "event", "level": "info", "foo": "bar"}
+        )
 
         assert dev._PlainStyles is plain_cr._styles
         assert "[info     ] event                          foo=bar" == rv
 
     def test_colorama_force_colors(self, styles, padded):
         """
-        If force_colors is True, use colors even if
-        the destination is non-tty.
+        If force_colors is True, use colors even if the destination is non-tty.
         """
         cr = dev.ConsoleRenderer(
-            colors=dev._has_colorama, force_colors=dev._has_colorama)
+            colors=dev._has_colorama, force_colors=dev._has_colorama
+        )
 
-        rv = cr(None, None, {
-            "event": "test", "level": "critical", "foo": "bar"
-        })
+        rv = cr(
+            None, None, {"event": "test", "level": "critical", "foo": "bar"}
+        )
 
+        # fmt: off
         assert (
             "[" + dev.RED + styles.bright +
             dev._pad("critical", cr._longest_level) +
@@ -274,6 +296,7 @@ class TestConsoleRenderer(object):
             styles.kv_key + "foo" + styles.reset + "=" +
             styles.kv_value + "bar" + styles.reset
         ) == rv
+        # fmt: on
 
         assert not dev._has_colorama or dev._ColorfulStyles is cr._styles
 
@@ -283,10 +306,8 @@ class TestConsoleRenderer(object):
         repr_native_str=False doesn't repr on native strings.  "event" is
         never repr'ed.
         """
-        rv = dev.ConsoleRenderer(
-            colors=False, repr_native_str=rns)(None, None, {
-                "event": "哈", "key": 42, "key2": "哈",
-            }
+        rv = dev.ConsoleRenderer(colors=False, repr_native_str=rns)(
+            None, None, {"event": "哈", "key": 42, "key2": "哈"}
         )
 
         cnt = rv.count("哈")
@@ -294,3 +315,42 @@ class TestConsoleRenderer(object):
             assert 1 == cnt
         else:
             assert 2 == cnt
+
+    @pytest.mark.parametrize("repr_native_str", [True, False])
+    @pytest.mark.parametrize("force_colors", [True, False])
+    @pytest.mark.parametrize("proto", range(pickle.HIGHEST_PROTOCOL))
+    def test_pickle(self, repr_native_str, force_colors, proto):
+        """
+        ConsoleRenderer can be pickled and unpickled.
+        """
+        r = dev.ConsoleRenderer(
+            repr_native_str=repr_native_str, force_colors=force_colors
+        )
+
+        assert r(None, None, {"event": "foo"}) == pickle.loads(
+            pickle.dumps(r, proto)
+        )(None, None, {"event": "foo"})
+
+
+class TestSetExcInfo(object):
+    def test_wrong_name(self):
+        """
+        Do nothing if name is not exception.
+        """
+        assert {} == dev.set_exc_info(None, "foo", {})
+
+    @pytest.mark.parametrize("ei", [False, None, ()])
+    def test_already_set(self, ei):
+        """
+        Do nothing if exc_info is already set.
+        """
+        assert {"exc_info": ei} == dev.set_exc_info(
+            None, "foo", {"exc_info": ei}
+        )
+
+    def test_set_it(self):
+        """
+        Set exc_info to True if its not set and if the method name is
+        exception.
+        """
+        assert {"exc_info": True} == dev.set_exc_info(None, "exception", {})
