@@ -8,6 +8,7 @@ from __future__ import absolute_import, division, print_function
 
 import datetime
 import json
+import pickle
 import sys
 
 import pytest
@@ -18,9 +19,16 @@ from freezegun import freeze_time
 import structlog
 
 from structlog.processors import (
-    ExceptionPrettyPrinter, JSONRenderer, KeyValueRenderer, StackInfoRenderer,
-    TimeStamper, UnicodeDecoder, UnicodeEncoder, _figure_out_exc_info,
-    _json_fallback_handler, format_exc_info
+    ExceptionPrettyPrinter,
+    JSONRenderer,
+    KeyValueRenderer,
+    StackInfoRenderer,
+    TimeStamper,
+    UnicodeDecoder,
+    UnicodeEncoder,
+    _figure_out_exc_info,
+    _json_fallback_handler,
+    format_exc_info,
 )
 from structlog.threadlocal import wrap_dict
 
@@ -43,73 +51,76 @@ class TestKeyValueRenderer(object):
         """
         Keys are sorted if sort_keys is set.
         """
-        assert (
-            r"a=<A(\o/)> b=[3, 4] x=7 y='test' z=(1, 2)" ==
-            KeyValueRenderer(sort_keys=True)(None, None, event_dict)
-        )
+        rv = KeyValueRenderer(sort_keys=True)(None, None, event_dict)
+
+        assert r"a=<A(\o/)> b=[3, 4] x=7 y='test' z=(1, 2)" == rv
 
     def test_order_complete(self, event_dict):
         """
         Orders keys according to key_order.
         """
-        assert (
-            r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" ==
-            KeyValueRenderer(key_order=['y', 'b', 'a', 'z', 'x'])
-            (None, None, event_dict)
+        rv = KeyValueRenderer(key_order=["y", "b", "a", "z", "x"])(
+            None, None, event_dict
         )
 
+        assert r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" == rv
+
     def test_order_missing(self, event_dict):
         """
         Missing keys get rendered as None.
         """
-        assert (
-            r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" ==
-            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'])
-            (None, None, event_dict)
+        rv = KeyValueRenderer(key_order=["c", "y", "b", "a", "z", "x"])(
+            None, None, event_dict
         )
 
+        assert r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" == rv
+
     def test_order_missing_dropped(self, event_dict):
         """
         Missing keys get dropped
         """
-        assert (
-            r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" ==
-            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'],
-                             drop_missing=True)
-            (None, None, event_dict)
-        )
+        rv = KeyValueRenderer(
+            key_order=["c", "y", "b", "a", "z", "x"], drop_missing=True
+        )(None, None, event_dict)
+
+        assert r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7" == rv
 
     def test_order_extra(self, event_dict):
         """
         Extra keys get sorted if sort_keys=True.
         """
-        event_dict['B'] = 'B'
-        event_dict['A'] = 'A'
+        event_dict["B"] = "B"
+        event_dict["A"] = "A"
+
+        rv = KeyValueRenderer(
+            key_order=["c", "y", "b", "a", "z", "x"], sort_keys=True
+        )(None, None, event_dict)
+
         assert (
-            r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7 A='A' B='B'" ==
-            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'],
-                             sort_keys=True)
-            (None, None, event_dict)
-        )
+            r"c=None y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7 A='A' B='B'"
+        ) == rv
 
     def test_order_sorted_missing_dropped(self, event_dict):
         """
         Keys get sorted if sort_keys=True and extras get dropped.
         """
-        event_dict['B'] = 'B'
-        event_dict['A'] = 'A'
-        assert (
-            r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7 A='A' B='B'" ==
-            KeyValueRenderer(key_order=['c', 'y', 'b', 'a', 'z', 'x'],
-                             sort_keys=True, drop_missing=True)
-            (None, None, event_dict)
-        )
+        event_dict["B"] = "B"
+        event_dict["A"] = "A"
+
+        rv = KeyValueRenderer(
+            key_order=["c", "y", "b", "a", "z", "x"],
+            sort_keys=True,
+            drop_missing=True,
+        )(None, None, event_dict)
+
+        assert r"y='test' b=[3, 4] a=<A(\o/)> z=(1, 2) x=7 A='A' B='B'" == rv
 
     def test_random_order(self, event_dict):
         """
         No special ordering doesn't blow up.
         """
         rv = KeyValueRenderer()(None, None, event_dict)
+
         assert isinstance(rv, str)
 
     @pytest.mark.parametrize("rns", [True, False])
@@ -117,9 +128,9 @@ class TestKeyValueRenderer(object):
         """
         repr_native_str=False doesn't repr on native strings.
         """
-        rv = KeyValueRenderer(repr_native_str=rns)(None, None, {
-            "event": "哈", "key": 42, "key2": "哈",
-        })
+        rv = KeyValueRenderer(repr_native_str=rns)(
+            None, None, {"event": "哈", "key": 42, "key2": "哈"}
+        )
 
         cnt = rv.count("哈")
         if rns and six.PY2:
@@ -133,26 +144,32 @@ class TestJSONRenderer(object):
         """
         Renders a predictable JSON string.
         """
+        rv = JSONRenderer(sort_keys=True)(None, None, event_dict)
+
         assert (
-            r'{"a": "<A(\\o/)>", "b": [3, 4], "x": 7, "y": "test", "z": '
-            r'[1, 2]}' ==
-            JSONRenderer(sort_keys=True)(None, None, event_dict)
-        )
+            r'{"a": "<A(\\o/)>", "b": [3, 4], "x": 7, '
+            r'"y": "test", "z": '
+            r"[1, 2]}"
+        ) == rv
 
     def test_FallbackEncoder_handles_ThreadLocalDictWrapped_dicts(self):
         """
         Our fallback handling handles properly ThreadLocalDictWrapper values.
         """
-        s = json.dumps(wrap_dict(dict)({'a': 42}),
-                       default=_json_fallback_handler)
+        s = json.dumps(
+            wrap_dict(dict)({"a": 42}), default=_json_fallback_handler
+        )
+
         assert '{"a": 42}' == s
 
     def test_FallbackEncoder_falls_back(self):
         """
         The fallback handler uses repr if it doesn't know the type.
         """
-        s = json.dumps({'date': datetime.date(1980, 3, 25)},
-                       default=_json_fallback_handler)
+        s = json.dumps(
+            {"date": datetime.date(1980, 3, 25)},
+            default=_json_fallback_handler,
+        )
 
         assert '{"date": "datetime.date(1980, 3, 25)"}' == s
 
@@ -165,6 +182,15 @@ class TestJSONRenderer(object):
 
         assert {"a": 42} == jr(None, None, obj)
 
+    def test_custom_fallback(self):
+        """
+        A custom fallback handler can be used.
+        """
+        jr = JSONRenderer(default=lambda x: repr(x)[::-1])
+        d = {"date": datetime.date(1980, 3, 25)}
+
+        assert '{"date": ")52 ,3 ,0891(etad.emitetad"}' == jr(None, None, d)
+
     @pytest.mark.skipif(simplejson is None, reason="simplejson is missing.")
     def test_simplejson(self, event_dict):
         """
@@ -173,11 +199,16 @@ class TestJSONRenderer(object):
         jr = JSONRenderer(serializer=simplejson.dumps)
 
         assert {
-            'a': '<A(\\o/)>', 'b': [3, 4], 'x': 7, 'y': 'test', 'z': [1, 2]
+            "a": "<A(\\o/)>",
+            "b": [3, 4],
+            "x": 7,
+            "y": "test",
+            "z": [1, 2],
         } == json.loads(jr(None, None, event_dict))
 
-    @pytest.mark.skipif(rapidjson is None,
-                        reason="python-rapidjson is missing.")
+    @pytest.mark.skipif(
+        rapidjson is None, reason="python-rapidjson is missing."
+    )
     def test_rapidjson(self, event_dict):
         """
         Integration test with python-rapidjson.
@@ -185,7 +216,11 @@ class TestJSONRenderer(object):
         jr = JSONRenderer(serializer=rapidjson.dumps)
 
         assert {
-            'a': '<A(\\o/)>', 'b': [3, 4], 'x': 7, 'y': 'test', 'z': [1, 2]
+            "a": "<A(\\o/)>",
+            "b": [3, 4],
+            "x": 7,
+            "y": "test",
+            "z": [1, 2],
         } == json.loads(jr(None, None, event_dict))
 
 
@@ -210,7 +245,7 @@ class TestTimeStamper(object):
         # freezegun doesn't work with time.time. :(
         assert isinstance(d["timestamp"], float)
 
-    @freeze_time('1980-03-25 16:00:00')
+    @freeze_time("1980-03-25 16:00:00")
     def test_local(self):
         """
         Timestamp in local timezone work.  We can't add a timezone to the
@@ -221,7 +256,7 @@ class TestTimeStamper(object):
 
         assert "1980-03-25T16:00:00" == d["timestamp"]
 
-    @freeze_time('1980-03-25 16:00:00')
+    @freeze_time("1980-03-25 16:00:00")
     def test_formats(self):
         """
         The fmt string is respected.
@@ -231,7 +266,7 @@ class TestTimeStamper(object):
 
         assert "1980" == d["timestamp"]
 
-    @freeze_time('1980-03-25 16:00:00')
+    @freeze_time("1980-03-25 16:00:00")
     def test_adds_Z_to_iso(self):
         """
         stdlib's isoformat is buggy, so we fix it.
@@ -241,7 +276,7 @@ class TestTimeStamper(object):
 
         assert "1980-03-25T16:00:00Z" == d["timestamp"]
 
-    @freeze_time('1980-03-25 16:00:00')
+    @freeze_time("1980-03-25 16:00:00")
     def test_key_can_be_specified(self):
         """
         Timestamp is stored with the specified key.
@@ -251,16 +286,38 @@ class TestTimeStamper(object):
 
         assert "03" == d["month"]
 
+    @freeze_time("1980-03-25 16:00:00")
+    @pytest.mark.parametrize("fmt", [None, "%Y"])
+    @pytest.mark.parametrize("utc", [True, False])
+    @pytest.mark.parametrize("key", [None, "other-key"])
+    @pytest.mark.parametrize("proto", range(pickle.HIGHEST_PROTOCOL))
+    def test_pickle(self, fmt, utc, key, proto):
+        """
+        TimeStamper is serializable.
+        """
+        # UNIX timestamps must be UTC.
+        if fmt is None and not utc:
+            pytest.skip()
+
+        ts = TimeStamper()
+
+        assert ts(None, None, {}) == pickle.loads(pickle.dumps(ts, proto))(
+            None, None, {}
+        )
+
 
 class TestFormatExcInfo(object):
     def test_formats_tuple(self, monkeypatch):
         """
         If exc_info is a tuple, it is used.
         """
-        monkeypatch.setattr(structlog.processors,
-                            "_format_exception",
-                            lambda exc_info: exc_info)
+        monkeypatch.setattr(
+            structlog.processors,
+            "_format_exception",
+            lambda exc_info: exc_info,
+        )
         d = format_exc_info(None, None, {"exc_info": (None, None, 42)})
+
         assert {"exception": (None, None, 42)} == d
 
     def test_gets_exc_info_on_bool(self):
@@ -270,20 +327,23 @@ class TestFormatExcInfo(object):
         # monkeypatching sys.exc_info makes currently py.test return 1 on
         # success.
         try:
-            raise ValueError('test')
+            raise ValueError("test")
         except ValueError:
             d = format_exc_info(None, None, {"exc_info": True})
+
         assert "exc_info" not in d
-        assert "raise ValueError('test')\nValueError: test" in d["exception"]
+        assert 'raise ValueError("test")\nValueError: test' in d["exception"]
 
     @py3_only
     def test_exception_on_py3(self, monkeypatch):
         """
         Passing excetions as exc_info is valid on Python 3.
         """
-        monkeypatch.setattr(structlog.processors,
-                            "_format_exception",
-                            lambda exc_info: exc_info)
+        monkeypatch.setattr(
+            structlog.processors,
+            "_format_exception",
+            lambda exc_info: exc_info,
+        )
         try:
             raise ValueError("test")
         except ValueError as e:
@@ -297,9 +357,10 @@ class TestFormatExcInfo(object):
         """
         If an Exception is missing a traceback, render it anyway.
         """
-        rv = format_exc_info(None, None, {
-            "exc_info": Exception("no traceback!")
-        })
+        rv = format_exc_info(
+            None, None, {"exc_info": Exception("no traceback!")}
+        )
+
         assert {"exception": "Exception: no traceback!"} == rv
 
 
@@ -309,6 +370,7 @@ class TestUnicodeEncoder(object):
         Unicode strings get encoded (as UTF-8 by default).
         """
         ue = UnicodeEncoder()
+
         assert {"foo": b"b\xc3\xa4r"} == ue(None, None, {"foo": u"b\xe4r"})
 
     def test_passes_arguments(self):
@@ -316,6 +378,7 @@ class TestUnicodeEncoder(object):
         Encoding options are passed into the encoding call.
         """
         ue = UnicodeEncoder("latin1", "xmlcharrefreplace")
+
         assert {"foo": b"&#8211;"} == ue(None, None, {"foo": u"\u2013"})
 
     def test_bytes_nop(self):
@@ -323,6 +386,7 @@ class TestUnicodeEncoder(object):
         If the string is already bytes, don't do anything.
         """
         ue = UnicodeEncoder()
+
         assert {"foo": b"b\xc3\xa4r"} == ue(None, None, {"foo": b"b\xc3\xa4r"})
 
 
@@ -332,6 +396,7 @@ class TestUnicodeDecoder(object):
         Byte strings get decoded (as UTF-8 by default).
         """
         ud = UnicodeDecoder()
+
         assert {"foo": u"b\xe4r"} == ud(None, None, {"foo": b"b\xc3\xa4r"})
 
     def test_passes_arguments(self):
@@ -339,6 +404,7 @@ class TestUnicodeDecoder(object):
         Encoding options are passed into the encoding call.
         """
         ud = UnicodeDecoder("utf-8", "ignore")
+
         assert {"foo": u""} == ud(None, None, {"foo": b"\xa1\xa4"})
 
     def test_bytes_nop(self):
@@ -346,6 +412,7 @@ class TestUnicodeDecoder(object):
         If the value is already unicode, don't do anything.
         """
         ud = UnicodeDecoder()
+
         assert {"foo": u"b\u2013r"} == ud(None, None, {"foo": u"b\u2013r"})
 
 
@@ -355,6 +422,7 @@ class TestExceptionPrettyPrinter(object):
         If no file is supplied, use stdout.
         """
         epp = ExceptionPrettyPrinter()
+
         assert sys.stdout is epp._file
 
     def test_prints_exception(self, sio):
@@ -366,12 +434,13 @@ class TestExceptionPrettyPrinter(object):
         try:
             raise ValueError
         except ValueError:
-            ed = format_exc_info(None, None, {'exc_info': True})
+            ed = format_exc_info(None, None, {"exc_info": True})
         epp(None, None, ed)
 
         out = sio.getvalue()
-        assert 'test_prints_exception' in out
-        assert 'raise ValueError' in out
+
+        assert "test_prints_exception" in out
+        assert "raise ValueError" in out
 
     def test_removes_exception_after_printing(self, sio):
         """
@@ -381,10 +450,13 @@ class TestExceptionPrettyPrinter(object):
         try:
             raise ValueError
         except ValueError:
-            ed = format_exc_info(None, None, {'exc_info': True})
-        assert 'exception' in ed
+            ed = format_exc_info(None, None, {"exc_info": True})
+
+        assert "exception" in ed
+
         new_ed = epp(None, None, ed)
-        assert 'exception' not in new_ed
+
+        assert "exception" not in new_ed
 
     def test_handles_exc_info(self, sio):
         """
@@ -394,11 +466,12 @@ class TestExceptionPrettyPrinter(object):
         try:
             raise ValueError
         except ValueError:
-            epp(None, None, {'exc_info': True})
+            epp(None, None, {"exc_info": True})
 
         out = sio.getvalue()
-        assert 'test_handles_exc_info' in out
-        assert 'raise ValueError' in out
+
+        assert "test_handles_exc_info" in out
+        assert "raise ValueError" in out
 
     def test_removes_exc_info_after_printing(self, sio):
         """
@@ -408,8 +481,9 @@ class TestExceptionPrettyPrinter(object):
         try:
             raise ValueError
         except ValueError:
-            ed = epp(None, None, {'exc_info': True})
-        assert 'exc_info' not in ed
+            ed = epp(None, None, {"exc_info": True})
+
+        assert "exc_info" not in ed
 
     def test_nop_if_no_exception(self, sio):
         """
@@ -417,7 +491,8 @@ class TestExceptionPrettyPrinter(object):
         """
         epp = ExceptionPrettyPrinter(sio)
         epp(None, None, {})
-        assert '' == sio.getvalue()
+
+        assert "" == sio.getvalue()
 
     def test_own_exc_info(self, sio):
         """
@@ -430,6 +505,7 @@ class TestExceptionPrettyPrinter(object):
             ei = sys.exc_info()
 
         epp(None, None, {"exc_info": ei})
+
         assert "XXX" in sio.getvalue()
 
     @py3_only
@@ -442,6 +518,7 @@ class TestExceptionPrettyPrinter(object):
             raise ValueError("XXX")
         except ValueError as e:
             epp(None, None, {"exc_info": e})
+
         assert "XXX" in sio.getvalue()
 
 
@@ -455,31 +532,32 @@ class TestStackInfoRenderer(object):
         """
         The `stack_info` key is removed from `event_dict`.
         """
-        ed = sir(None, None, {'stack_info': True})
-        assert 'stack_info' not in ed
+        ed = sir(None, None, {"stack_info": True})
+
+        assert "stack_info" not in ed
 
     def test_adds_stack_if_asked(self, sir):
         """
         If `stack_info` is true, `stack` is added.
         """
-        ed = sir(None, None, {'stack_info': True})
-        assert 'stack' in ed
+        ed = sir(None, None, {"stack_info": True})
+
+        assert "stack" in ed
 
     def test_renders_correct_stack(self, sir):
-        ed = sir(None, None, {'stack_info': True})
-        assert "ed = sir(None, None, {'stack_info': True})" in ed['stack']
+        ed = sir(None, None, {"stack_info": True})
+
+        assert 'ed = sir(None, None, {"stack_info": True})' in ed["stack"]
 
 
 class TestFigureOutExcInfo(object):
-    @pytest.mark.parametrize('true_value', [
-        True, 1, 1.1
-    ])
+    @pytest.mark.parametrize("true_value", [True, 1, 1.1])
     def test_obtains_exc_info_on_True(self, true_value):
         """
         If the passed argument evaluates to True obtain exc_info ourselves.
         """
         try:
-            0/0
+            0 / 0
         except Exception:
             assert sys.exc_info() == _figure_out_exc_info(true_value)
         else:
@@ -492,4 +570,5 @@ class TestFigureOutExcInfo(object):
         traceback.
         """
         e = ValueError()
+
         assert (e.__class__, e, None) == _figure_out_exc_info(e)
