@@ -12,8 +12,17 @@ import pytest
 
 from structlog._base import BoundLoggerBase
 from structlog._config import wrap_logger
-from structlog._loggers import ReturnLogger
-from structlog.threadlocal import as_immutable, tmp_bind, wrap_dict
+from structlog.testing import ReturnLogger
+from structlog.threadlocal import (
+    as_immutable,
+    bind_threadlocal,
+    clear_threadlocal,
+    merge_threadlocal,
+    merge_threadlocal_context,
+    tmp_bind,
+    unbind_threadlocal,
+    wrap_dict,
+)
 
 
 try:
@@ -31,11 +40,11 @@ def D():
 
 
 @pytest.fixture
-def log():
+def log(logger):
     """
     Returns a ReturnLogger with a freshly wrapped OrderedDict.
     """
-    return wrap_logger(logger(), context_class=wrap_dict(OrderedDict))
+    return wrap_logger(logger, context_class=wrap_dict(OrderedDict))
 
 
 @pytest.fixture
@@ -53,11 +62,13 @@ class TestTmpBind(object):
         tmp_bind does not modify the thread-local state.
         """
         log = log.bind(y=23)
-        with tmp_bind(log, x=42, y='foo') as tmp_log:
-            assert {
-                'y': 'foo', 'x': 42
-            } == tmp_log._context._dict == log._context._dict
-        assert {'y': 23} == log._context._dict
+        with tmp_bind(log, x=42, y="foo") as tmp_log:
+            assert (
+                {"y": "foo", "x": 42}
+                == tmp_log._context._dict
+                == log._context._dict
+            )
+        assert {"y": 23} == log._context._dict
 
     def test_bind_exc(self, log):
         """
@@ -65,12 +76,15 @@ class TestTmpBind(object):
         """
         log = log.bind(y=23)
         with pytest.raises(ValueError):
-            with tmp_bind(log, x=42, y='foo') as tmp_log:
-                assert {
-                           'y': 'foo', 'x': 42
-                       } == tmp_log._context._dict == log._context._dict
+            with tmp_bind(log, x=42, y="foo") as tmp_log:
+                assert (
+                    {"y": "foo", "x": 42}
+                    == tmp_log._context._dict
+                    == log._context._dict
+                )
                 raise ValueError
-        assert {'y': 23} == log._context._dict
+
+        assert {"y": 23} == log._context._dict
 
 
 class TestAsImmutable(object):
@@ -80,10 +94,13 @@ class TestAsImmutable(object):
         """
         log = log.new(x=42)
         il = as_immutable(log)
+
         assert isinstance(il._context, dict)
+
         il = il.bind(y=23)
-        assert {'x': 42, 'y': 23} == il._context
-        assert {'x': 42} == log._context._dict
+
+        assert {"x": 42, "y": 23} == il._context
+        assert {"x": 42} == log._context._dict
 
     def test_converts_proxy(self, log):
         """
@@ -91,6 +108,7 @@ class TestAsImmutable(object):
         logger.
         """
         il = as_immutable(log)
+
         assert isinstance(il._context, dict)
         assert isinstance(il, BoundLoggerBase)
 
@@ -99,6 +117,7 @@ class TestAsImmutable(object):
         as_immutable on an as_immutable logger works.
         """
         il = as_immutable(log)
+
         assert isinstance(as_immutable(il), BoundLoggerBase)
 
 
@@ -110,74 +129,89 @@ class TestThreadLocalDict(object):
         """
         D1 = wrap_dict(dict)
         D2 = wrap_dict(dict)
+
         assert D1 != D2
         assert D1 is not D2
+
         D1.x = 42
         D2.x = 23
+
         assert D1.x != D2.x
 
-    @pytest.mark.skipif(greenlet is not None,
-                        reason="Don't mix threads and greenlets.")
+    @pytest.mark.skipif(
+        greenlet is not None, reason="Don't mix threads and greenlets."
+    )
     def test_is_thread_local(self, D):
         """
         The context is *not* shared between threads.
         """
+
         class TestThread(threading.Thread):
             def __init__(self, d):
                 self._d = d
                 threading.Thread.__init__(self)
 
             def run(self):
-                assert 'tl' not in self._d._dict
-                self._d['tl'] = 23
+                assert "tl" not in self._d._dict
+
+                self._d["tl"] = 23
+
         d = wrap_dict(dict)()
-        d['tl'] = 42
+        d["tl"] = 42
         t = TestThread(d)
         t.start()
         t.join()
-        assert 42 == d._dict['tl']
+
+        assert 42 == d._dict["tl"]
 
     def test_context_is_global_to_thread(self, D):
         """
         The context is shared between all instances of a wrapped class.
         """
-        d1 = D({'a': 42})
-        d2 = D({'b': 23})
+        d1 = D({"a": 42})
+        d2 = D({"b": 23})
         d3 = D()
-        assert {'a': 42, 'b': 23} == d1._dict == d2._dict == d3._dict
+
+        assert {"a": 42, "b": 23} == d1._dict == d2._dict == d3._dict
         assert d1 == d2 == d3
+
         D_ = wrap_dict(dict)
-        d_ = D_({'a': 42, 'b': 23})
+        d_ = D_({"a": 42, "b": 23})
+
         assert d1 != d_
 
     def test_init_with_itself_works(self, D):
         """
         Initializing with an instance of the wrapped class will use its values.
         """
-        d = D({'a': 42})
-        assert {'a': 42, 'b': 23} == D(d, b=23)._dict
+        d = D({"a": 42})
+
+        assert {"a": 42, "b": 23} == D(d, b=23)._dict
 
     def test_iter_works(self, D):
         """
         ___iter__ is proxied to the wrapped class.
         """
-        d = D({'a': 42})
-        assert ['a'] == list(iter(d))
+        d = D({"a": 42})
+
+        assert ["a"] == list(iter(d))
 
     def test_non_dunder_proxy_works(self, D):
         """
         Calls to a non-dunder method get proxied to the wrapped class.
         """
-        d = D({'a': 42})
+        d = D({"a": 42})
         d.clear()
+
         assert 0 == len(d)
 
     def test_repr(self, D):
         """
         ___repr__ takes the repr of the wrapped class into account.
         """
-        r = repr(D({'a': 42}))
-        assert r.startswith('<WrappedDict-')
+        r = repr(D({"a": 42}))
+
+        assert r.startswith("<WrappedDict-")
         assert r.endswith("({'a': 42})>")
 
     @pytest.mark.skipif(greenlet is None, reason="Needs greenlet.")
@@ -186,13 +220,15 @@ class TestThreadLocalDict(object):
         Context is shared between greenlets.
         """
         d = wrap_dict(dict)()
-        d['switch'] = 42
+        d["switch"] = 42
 
         def run():
-            assert 'x' not in d._dict
-            d['switch'] = 23
+            assert "x" not in d._dict
+
+            d["switch"] = 23
 
         greenlet.greenlet(run).switch()
+
         assert 42 == d._dict["switch"]
 
     def test_delattr(self, D):
@@ -200,8 +236,10 @@ class TestThreadLocalDict(object):
         ___delattr__ is proxied to the wrapped class.
         """
         d = D()
-        d['delattr'] = 42
+        d["delattr"] = 42
+
         assert 42 == d._dict["delattr"]
+
         del d.__class__._tl.dict_
 
     def test_delattr_missing(self, D):
@@ -209,8 +247,10 @@ class TestThreadLocalDict(object):
         __delattr__ on an inexisting attribute raises AttributeError.
         """
         d = D()
+
         with pytest.raises(AttributeError) as e:
             d._tl.__delattr__("does_not_exist")
+
         assert "does_not_exist" == e.value.args[0]
 
     def test_del(self, D):
@@ -219,8 +259,11 @@ class TestThreadLocalDict(object):
         """
         d = D()
         d["del"] = 13
+
         assert 13 == d._dict["del"]
+
         del d["del"]
+
         assert "del" not in d._dict
 
     def test_new_class(self, D):
@@ -228,3 +271,67 @@ class TestThreadLocalDict(object):
         The context of a new wrapped class is empty.
         """
         assert 0 == len(D())
+
+
+class TestNewThreadLocal(object):
+    def test_alias(self):
+        """
+        We're keeping the old alias around.
+        """
+        assert merge_threadlocal_context is merge_threadlocal
+
+    def test_bind_and_merge(self):
+        """
+        Binding a variable causes it to be included in the result of
+        merge_threadlocal.
+        """
+        bind_threadlocal(a=1)
+
+        assert {"a": 1, "b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_clear(self):
+        """
+        The thread-local context can be cleared, causing any previously bound
+        variables to not be included in merge_threadlocal's result.
+        """
+        bind_threadlocal(a=1)
+        clear_threadlocal()
+
+        assert {"b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_merge_works_without_bind(self):
+        """
+        merge_threadlocal returns values as normal even when there has
+        been no previous calls to bind_threadlocal.
+        """
+        assert {"b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_multiple_binds(self):
+        """
+        Multiple calls to bind_threadlocal accumulate values instead of
+        replacing them.
+        """
+        bind_threadlocal(a=1, b=2)
+        bind_threadlocal(c=3)
+
+        assert {"a": 1, "b": 2, "c": 3} == merge_threadlocal(
+            None, None, {"b": 2}
+        )
+
+    def test_unbind_threadlocal(self):
+        """
+        Test that unbinding from threadlocal works for keys that exist
+        and does not raise error when they do not exist.
+        """
+
+        clear_threadlocal()
+        bind_threadlocal(a=234, b=34)
+        assert {"a": 234, "b": 34} == merge_threadlocal_context(None, None, {})
+
+        unbind_threadlocal("a")
+
+        assert {"b": 34} == merge_threadlocal_context(None, None, {})
+
+        unbind_threadlocal("non-existing-key")
+
+        assert {"b": 34} == merge_threadlocal_context(None, None, {})
